/////////////////////////////////////////////////////////////////////////////////////

// DirectInput

/////////////////////////////////////////////////////////////////////////////////////



#define WIN32_LEAN_AND_MEAN

#include <windows.h>

#include <dinput.h>

#include <mmsystem.h>

#include "input.h"



#define DI_Error(a,b) {}

#undef RELEASE

#ifdef __cplusplus

#define RELEASE(x) if (x != NULL) {x->Release(); x = NULL;}

#else

#define RELEASE(x) if (x != NULL) {x->lpVtbl->Release(x); x = NULL;}

#endif



extern HWND hMainWnd;



LPDIRECTINPUT        lpDirectInput = 0;

LPDIRECTINPUTDEVICE  lpMouse       = 0;

LPDIRECTINPUTDEVICE2 lpJoystick    = 0;

LPDIRECTINPUTDEVICE  lpKeyboard    = 0;

static unsigned char diKeyState[256];



//==========================================================================



int InitDirectInput()

{

	HRESULT hresult;

	LPDIRECTINPUTDEVICE lpJoy;

	DIPROPRANGE diprg; 

	DIPROPDWORD dipdw;

	HINSTANCE hInst = GetModuleHandle(NULL);



	memset(diKeyState, 0, sizeof(diKeyState));

	hresult = DirectInputCreate(hInst, DIRECTINPUT_VERSION, &lpDirectInput, NULL);

	if (hresult != DI_OK)

	{

		DI_Error(hresult, "DirectInputCreate");

		return 0;

	}

#ifndef _DEBUG

	hresult = lpDirectInput->lpVtbl->CreateDevice(lpDirectInput, &GUID_SysMouse, &lpMouse, NULL);

	if (hresult != DI_OK)

	{

		DI_Error(hresult, "CreateDevice (mouse)");

		ShutdownDirectInput();

		return 0;

	}

#endif

	hresult = lpDirectInput->lpVtbl->CreateDevice(lpDirectInput, &GUID_Joystick, &lpJoy, NULL);

	if (hresult == DI_OK)

	{

		hresult = lpJoy->lpVtbl->QueryInterface(lpJoy,&IID_IDirectInputDevice2,(LPVOID*)&lpJoystick);

		if (hresult != DI_OK)

		{

			DI_Error( hresult, "CreateDevice (real joystick)");

		}

		RELEASE(lpJoy);

	}



	hresult = lpDirectInput->lpVtbl->CreateDevice(lpDirectInput, &GUID_SysKeyboard, &lpKeyboard, NULL);

	if (hresult != DI_OK)

	{

		DI_Error(hresult, "CreateDevice (keyboard)");

		ShutdownDirectInput();

		return 0;

	}

#ifndef _DEBUG

	hresult = lpMouse->lpVtbl->SetDataFormat(lpMouse, &c_dfDIMouse);

	if (hresult != DI_OK)

	{

		DI_Error( hresult, "SetDataFormat (mouse)");

		ShutdownDirectInput();

		return 0;

	}

#endif

	if (lpJoystick != 0)

	{

		hresult = lpJoystick->lpVtbl->SetDataFormat(lpJoystick, &c_dfDIJoystick);

		if (hresult != DI_OK)

		{

			DI_Error(hresult, "SetDataFormat (joystick)");

			ShutdownDirectInput();

			return 0;

		}

	}



	hresult = lpKeyboard->lpVtbl->SetDataFormat(lpKeyboard, &c_dfDIKeyboard);

	if (hresult != DI_OK)

	{

		DI_Error(hresult, "SetDataFormat (keyboard)");

		ShutdownDirectInput();

		return 0;

	}



	if (lpJoystick != 0)

	{

		hresult = lpJoystick->lpVtbl->SetCooperativeLevel(lpJoystick, hMainWnd, DISCL_EXCLUSIVE | DISCL_FOREGROUND);

		if (hresult != DI_OK)

		{

			DI_Error( hresult, "SetCooperativeLevel (joystick)");

			ShutdownDirectInput();

			return 0;

		}

		else

		{

			diprg.diph.dwSize		= sizeof(diprg); 

			diprg.diph.dwHeaderSize = sizeof(diprg.diph); 

			diprg.diph.dwObj		= DIJOFS_X; 

			diprg.diph.dwHow		= DIPH_BYOFFSET; 

			diprg.lMin				= -1000; 

			diprg.lMax				= +1000; 

			lpJoystick->lpVtbl->SetProperty(lpJoystick, DIPROP_RANGE, &diprg.diph);

			diprg.diph.dwObj		= DIJOFS_Y;

			lpJoystick->lpVtbl->SetProperty(lpJoystick, DIPROP_RANGE, &diprg.diph);



			dipdw.diph.dwSize		= sizeof(DIPROPDWORD);

			dipdw.diph.dwHeaderSize = sizeof(dipdw.diph);

			dipdw.diph.dwHow		= DIPH_BYOFFSET;

			dipdw.dwData			= 1500;

			dipdw.diph.dwObj		= DIJOFS_X;

			lpJoystick->lpVtbl->SetProperty(lpJoystick, DIPROP_DEADZONE, &dipdw.diph);

			dipdw.diph.dwObj		= DIJOFS_Y;

			lpJoystick->lpVtbl->SetProperty(lpJoystick, DIPROP_DEADZONE, &dipdw.diph);

		}

	}

#ifndef _DEBUG

	hresult = lpMouse->lpVtbl->SetCooperativeLevel(lpMouse, hMainWnd, DISCL_EXCLUSIVE | DISCL_FOREGROUND);

	if (hresult != DI_OK)

	{

		DI_Error( hresult, "SetCooperativeLevel (mouse)");

		ShutdownDirectInput();

		return 0;

	}

#endif

	hresult = lpKeyboard->lpVtbl->SetCooperativeLevel(lpKeyboard, hMainWnd, DISCL_NONEXCLUSIVE | DISCL_FOREGROUND);

	if (hresult != DI_OK)

	{

		DI_Error( hresult, "SetCooperativeLevel (keyboard)");

		ShutdownDirectInput();

		return 0;

	}



#ifndef _DEBUG

	hresult = lpMouse->lpVtbl->Acquire(lpMouse);

#endif

	if (lpJoystick != 0)

		hresult = lpJoystick->lpVtbl->Acquire(lpJoystick);

	hresult = lpKeyboard->lpVtbl->Acquire(lpKeyboard);



	return 1;

}

   

void ShutdownDirectInput()

{

	if (lpKeyboard != 0)

	{

		lpKeyboard->lpVtbl->Unacquire(lpKeyboard);

		lpKeyboard->lpVtbl->Release(lpKeyboard);

		lpKeyboard = 0;

	}



	if (lpJoystick != 0)

	{

		lpJoystick->lpVtbl->Unacquire(lpJoystick);

		lpJoystick->lpVtbl->Release(lpJoystick);

		lpJoystick = 0;

	}



	if (lpMouse != 0)

	{

		lpMouse->lpVtbl->Unacquire(lpMouse);

		lpMouse->lpVtbl->Release(lpMouse);

		lpMouse = 0;

	}



	RELEASE(lpDirectInput);

}



//==========================================================================



int S_JoystickExists(int joy)

{

	JOYINFOEX ji;



	ji.dwSize = sizeof(JOYINFOEX);

	ji.dwFlags = JOY_RETURNBUTTONS;



	if (joyGetPosEx(joy, &ji) == JOYERR_NOERROR)

		return 1;

	else

		return 0;

}



unsigned char S_PollJoyButtons()

{

	DIJOYSTATE diJoyState;

	HRESULT hresult;

	unsigned char ret = 0;



RetryJoy:;

	if (lpJoystick != 0)

	{

		hresult = lpJoystick->lpVtbl->Poll(lpJoystick);



		hresult = lpJoystick->lpVtbl->GetDeviceState(lpJoystick, sizeof(DIJOYSTATE), &diJoyState);

		if (hresult == DIERR_INPUTLOST)

		{

			hresult = lpJoystick->lpVtbl->Acquire(lpJoystick);

			if (SUCCEEDED(hresult))

				goto RetryJoy;

		}

		else if (hresult != DI_OK)

		{

			 DI_Error(hresult, "GetDeviceState (joystick)");

		}



		if (SUCCEEDED(hresult))

		{

			if (diJoyState.rgbButtons[0])

				ret |= 1;

			if (diJoyState.rgbButtons[1])

				ret |= 2;

			if (diJoyState.rgbButtons[2])

				ret |= 4;

			if (diJoyState.rgbButtons[3])

				ret |= 8;

		}

	}



	return ret;

}



void S_PollJoyPosition(unsigned short *xp, unsigned short *yp)

{

	DIJOYSTATE diJoyState;

	HRESULT hresult;



RetryJoy:;

	if (lpJoystick != 0)

	{

		hresult = lpJoystick->lpVtbl->Poll(lpJoystick);



		hresult = lpJoystick->lpVtbl->GetDeviceState(lpJoystick, sizeof(DIJOYSTATE), &diJoyState);

		if (hresult == DIERR_INPUTLOST)

		{

			hresult = lpJoystick->lpVtbl->Acquire(lpJoystick);

			if (SUCCEEDED(hresult))

				goto RetryJoy;

		}

		else if (hresult != DI_OK)

		{

			 DI_Error(hresult, "GetDeviceState (joystick)");

		}



		if (SUCCEEDED(hresult))

		{

			*xp = (unsigned short)diJoyState.lX+1000;

			*yp = (unsigned short)diJoyState.lY+1000;

		}

	}

}



//==========================================================================



static DIMOUSESTATE diMouseState;



int S_MouseExists()

{

	if (lpMouse == NULL)

		return 0;

	else

		return 1;

}



unsigned char S_PollMouseButtons()

{

	HRESULT hresult;

	unsigned char ret = 0;



RetryMouse:;

	hresult = lpMouse->lpVtbl->GetDeviceState(lpMouse, sizeof(DIMOUSESTATE), &diMouseState);

	

	if (hresult == DIERR_INPUTLOST)

	{

		hresult = lpMouse->lpVtbl->Acquire(lpMouse);

		if (SUCCEEDED(hresult))

			goto RetryMouse;

	}

	else if (hresult != DI_OK)

	{

		DI_Error(hresult, "GetDeviceState (mouse)");

	}

		

	if (SUCCEEDED(hresult))

	{

		if (diMouseState.rgbButtons[0]) ret |= 1;

		if (diMouseState.rgbButtons[1]) ret |= 2;

		if (diMouseState.rgbButtons[2]) ret |= 4;

	}



	return ret;

}



void S_PollMousePosition(int *xp, int *yp)

{

	*xp = diMouseState.lX*4;

	*yp = diMouseState.lY*4;

}



//==========================================================================



void IN_KeyEvent(unsigned char key);



void HandleKeyboard()

{

	HRESULT hresult;

	unsigned char k, keys[256];

	int i;

 

RetryKeyboard:;

	hresult = lpKeyboard->lpVtbl->GetDeviceState(lpKeyboard, sizeof(keys), &keys);

	if (hresult == DIERR_INPUTLOST)

	{

		hresult = lpKeyboard->lpVtbl->Acquire(lpKeyboard);

		if (SUCCEEDED(hresult))

			goto RetryKeyboard;

	}

	else if (hresult != DI_OK)

	{

		DI_Error(hresult, "GetDeviceState (keyboard)");

	}

	else

	{

		keys[DIK_RETURN] |= keys[DIK_NUMPADENTER];

		keys[DIK_LCONTROL] |= keys[DIK_RCONTROL];

		keys[DIK_SLASH]	|= keys[DIK_DIVIDE];

		keys[DIK_LMENU] |= keys[DIK_RMENU];

		keys[DIK_NUMPAD7] |= keys[DIK_HOME];

		keys[DIK_NUMPAD8] |= keys[DIK_UP];

		keys[DIK_NUMPAD9] |= keys[DIK_PRIOR];

		keys[DIK_NUMPAD4] |= keys[DIK_LEFT];

		keys[DIK_NUMPAD6] |= keys[DIK_RIGHT];

		keys[DIK_NUMPAD1] |= keys[DIK_END];

		keys[DIK_NUMPAD2] |= keys[DIK_DOWN];

		keys[DIK_NUMPAD3] |= keys[DIK_NEXT];

		keys[DIK_NUMPAD0] |= keys[DIK_INSERT];

		keys[DIK_DECIMAL] |= keys[DIK_DELETE];



		for (i = 0; i < 128; i++)

		{

			if (diKeyState[i] != keys[i])

			{

				diKeyState[i] = keys[i];

				k = (unsigned char)i;



				if ((keys[i] & 0x80) == 0)

					k |= 0x80;



				IN_KeyEvent(k);

			}

		}

	}

}

