//

// Dialog to choose the resolution

//



#define WIN32_LEAN_AND_MEAN

#include <windows.h>

#include <gl/gl.h>

#include <stdio.h>

#include "resource.h"

#include "system.h"

#include "setup.h"



void CreateTestWindow(int mode);

HWND hSetupDlg;

int ScreenMode;



LRESULT CALLBACK SetupDlgProc(HWND hWnd, UINT iMsg, WPARAM wParam, LPARAM lParam)

{

	switch (iMsg)

	{

		case WM_INITDIALOG:

		{

			DEVMODE devMode;

			int i, ret;

			char buf[64];

			HWND hList = GetDlgItem(hWnd, ID_LIST);

			hSetupDlg = hWnd;



			for (i = 0;;i++)

			{

				ret = EnumDisplaySettings(NULL, i, &devMode);



				if (ret == FALSE)

					break;



				sprintf(buf, "%dx%d - %d bits", devMode.dmPelsWidth, devMode.dmPelsHeight, devMode.dmBitsPerPel);



				ret = SendMessage(hList, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)buf);

				SendMessage(hList, LB_SETITEMDATA, ret, i);

			}



			ret = SendMessage(hList, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)"Window");

			SendMessage(hList, LB_SETITEMDATA, ret, -1);

			SendMessage(hList, LB_SETCURSEL, ret, 0);



		} break;



		case WM_COMMAND:

		{

			int ret;

			HWND hList = GetDlgItem(hWnd, ID_LIST);



			ret = SendMessage(hList, LB_GETCURSEL, 0, 0);

			ret = SendMessage(hList, LB_GETITEMDATA, ret, 0);



			if (LOWORD(wParam) == IDCANCEL)

				EndDialog(hWnd, 0);



			if (LOWORD(wParam) == IDOK)

			{

				ScreenMode = ret;

				EndDialog(hWnd, 1);

			}



			if (LOWORD(wParam) == ID_TEST)

				CreateTestWindow(ret);

		} break;

		

		default:

			return DefWindowProc(hWnd, iMsg, wParam, lParam);

	}



	return 0;

}



int ChangeResolution(int mode)

{

	DEVMODE devMode;

	int ret;



	ret = EnumDisplaySettings(NULL, mode, &devMode);

	ret = ChangeDisplaySettings(&devMode, CDS_FULLSCREEN);



	screen_width = devMode.dmPelsWidth;

	screen_height = devMode.dmPelsHeight;



	if (ret == DISP_CHANGE_SUCCESSFUL)

		return TRUE;



	// Might be running in Windows95, let's try without the hertz change

	devMode.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT; 

	ret = ChangeDisplaySettings(&devMode, CDS_FULLSCREEN); 

	if (ret == DISP_CHANGE_SUCCESSFUL)

		return TRUE;



	// Try with separate changes to bitdepth and resolution

	devMode.dmFields = DM_BITSPERPEL; 

	ret = ChangeDisplaySettings(&devMode, CDS_FULLSCREEN);

	if (ret == DISP_CHANGE_SUCCESSFUL)

	{

		devMode.dmFields = DM_PELSWIDTH | DM_PELSHEIGHT; 

		ret = ChangeDisplaySettings(&devMode, CDS_FULLSCREEN); 

		if (ret == DISP_CHANGE_SUCCESSFUL)

			return TRUE;

		ChangeDisplaySettings(NULL, 0);

	}



	if (hSetupDlg)

		SetDlgItemText(hSetupDlg, ID_INFO, "Failed to change video mode !");



	return FALSE; 

}



int DoSetupDlg()

{

	if (!DialogBox(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_SETUP), NULL, SetupDlgProc))

		return 0;

	hSetupDlg = 0;



	if (ScreenMode != -1)

		if (!ChangeResolution(ScreenMode))

			return 0;



	return 1;

}



LRESULT CALLBACK TestWndProc(HWND hWnd, UINT iMsg, WPARAM wParam, LPARAM lParam)

{

	switch(iMsg)

	{

		case WM_ERASEBKGND:

		{

			return TRUE;

		} break;



		case WM_CREATE:

		{

			PIXELFORMATDESCRIPTOR pfd = { sizeof(PIXELFORMATDESCRIPTOR), 1, 

				PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,

				PFD_TYPE_RGBA, 24, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,

				32, 0, 0, PFD_MAIN_PLANE, 0, 0, 0, 0 };

			int pixelformat;

			HDC hDC;

			HGLRC hRC;



			hDC = GetDC(hWnd);



			if ((pixelformat = ChoosePixelFormat(hDC, &pfd)) == 0)

			{

				MessageBox(NULL, "ChoosePixelFormat failed", "Error", MB_OK);

				ReleaseDC(hWnd, hDC);

				return TRUE;

			}



			if (SetPixelFormat(hDC, pixelformat, &pfd) == FALSE)

			{

				MessageBox(NULL, "SetPixelFormat failed", "Error", MB_OK);

				ReleaseDC(hWnd, hDC);

				return TRUE;

			}

			

			hRC = wglCreateContext(hDC);

			wglMakeCurrent(hDC, hRC);



			return TRUE;

		} break;



		case WM_DESTROY:

		{

			HDC hDC = wglGetCurrentDC();

			HGLRC hRC = wglGetCurrentContext();

			wglMakeCurrent(NULL, NULL);



			if (hRC)

				wglDeleteContext(hRC);

			if (hDC)

				ReleaseDC(hWnd, hDC);



			return TRUE;

		} break;



		default:

			return DefWindowProc(hWnd, iMsg, wParam, lParam);

	}



	return FALSE;

}



void CreateTestWindow(int mode)

{

	WNDCLASS wndclass;

	DWORD style;

	HWND hWnd;

	char tmp[64], info[256] = "";

	PIXELFORMATDESCRIPTOR pfd;



	wndclass.style		   = CS_OWNDC;

	wndclass.lpfnWndProc   = TestWndProc;

	wndclass.cbClsExtra    = 0;

	wndclass.cbWndExtra    = 0;

	wndclass.hInstance	   = GetModuleHandle(NULL);

	wndclass.hIcon		   = 0;//LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(ID_ICON));

	wndclass.hCursor	   = LoadCursor(NULL, IDC_WAIT);

	wndclass.hbrBackground = NULL;

	wndclass.lpszMenuName  = NULL;

	wndclass.lpszClassName = "OpenTestGLClass";

	RegisterClass(&wndclass);



	if (mode != -1)

		if (!ChangeResolution(mode))

			return;



	if (mode == -1)

		style = WS_OVERLAPPEDWINDOW|WS_CLIPSIBLINGS|WS_CLIPCHILDREN;

	else

		style = WS_POPUP|WS_MAXIMIZE|WS_CLIPSIBLINGS|WS_CLIPCHILDREN;



	hWnd = CreateWindow("OpenTestGLClass", "WolfGL Test", style,

		CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,

		NULL, NULL, GetModuleHandle(NULL), NULL);



	ShowWindow(hWnd, SW_SHOWDEFAULT);

	UpdateWindow(hWnd);

	SetFocus(hSetupDlg);



	DescribePixelFormat(wglGetCurrentDC(), GetPixelFormat(wglGetCurrentDC()), sizeof(PIXELFORMATDESCRIPTOR), &pfd);



	if (mode != -1)

	{

		DEVMODE devMode;

		EnumDisplaySettings(NULL, mode, &devMode);

		sprintf(info, "Mode: %dx%d - %d bits\r\n", devMode.dmPelsWidth, devMode.dmPelsHeight, devMode.dmBitsPerPel);

	}



	strcat(info, "Pixel Format: ");

	if (pfd.iPixelType == PFD_TYPE_RGBA)

		strcat(info, "RGBA ");

	else

		strcat(info, "Color Index ");



	if ((pfd.dwFlags & (PFD_GENERIC_ACCELERATED|PFD_GENERIC_FORMAT)) == 0)

		strcat(info, "(Installable Client Driver)\r\n");

	else if ((pfd.dwFlags & (PFD_GENERIC_ACCELERATED|PFD_GENERIC_FORMAT)) == (PFD_GENERIC_ACCELERATED|PFD_GENERIC_FORMAT))

		strcat(info, "(Mini-Client Driver)\r\n");

	else if ((pfd.dwFlags & (PFD_GENERIC_ACCELERATED|PFD_GENERIC_FORMAT)) == PFD_GENERIC_FORMAT)

		strcat(info, "(Generic Software Driver)\r\n");

	else if ((pfd.dwFlags & (PFD_GENERIC_ACCELERATED|PFD_GENERIC_FORMAT)) == PFD_GENERIC_ACCELERATED)

		strcat(info, "(Unknown Driver Type)\r\n");



	sprintf(tmp, "Color bits: %d, Depth Buffer: %d bits - ", pfd.cColorBits, pfd.cDepthBits);

	strcat(info, tmp);

	if (pfd.dwFlags & PFD_DOUBLEBUFFER)

		strcat(info, "Double Buffer");

	else

		strcat(info, "Single Buffer");

	strcat(info, "\r\nOpenGL Version ");

	strcat(info, glGetString(GL_VERSION));

	strcat(info, " (");

	strcat(info, glGetString(GL_RENDERER));

	strcat(info, " - ");

	strcat(info, glGetString(GL_VENDOR));

	strcat(info, ")");



	DestroyWindow(hWnd);



	if (mode != -1)

		ChangeDisplaySettings(NULL, 0);



	SetDlgItemText(hSetupDlg, ID_INFO, info);

}



