/////////////////////////////////////////////////////////////////////////////////////

// DIRECTSOUND - Sound effects

/////////////////////////////////////////////////////////////////////////////////////



#include <dsound.h>

#include "wl_def.h"



//==========================================================================



#define DS_Error(a,b) {}

#define NUM_DSBUFFERS 256



#undef RELEASE

#ifdef __cplusplus

#define RELEASE(x) if (x != NULL) {x->Release(); x = NULL;}

#else

#define RELEASE(x) if (x != NULL) {x->lpVtbl->Release(x); x = NULL;}

#endif



extern HWND hMainWnd;



LPDIRECTSOUND        lpDS;

LPDIRECTSOUNDBUFFER  lpDSPrimary;

LPDIRECTSOUNDBUFFER  lpDSBuffer[NUM_DSBUFFERS];



int DigitalDevice;



//==========================================================================



BOOL SetupDirectSound()

{

	HRESULT 	   hresult;

	int 		   buff;

	DSBUFFERDESC   dsbdesc;

	WAVEFORMATEX   wfx;



	// Create an instance of DirectSound

	hresult = DirectSoundCreate(NULL, &lpDS, NULL);

	if (hresult != DS_OK)

	{

		DS_Error(hresult, "DirectSoundCreate");

		for (buff = 0; buff < NUM_DSBUFFERS; buff++)

			lpDSBuffer[buff] = 0;



		DigitalDevice = false;



		return FALSE;

	}

	DigitalDevice = true;



	// Set the cooperative level so it doesn't get confused

	hresult = lpDS->lpVtbl->SetCooperativeLevel(lpDS, hMainWnd, DSSCL_EXCLUSIVE);

	if (hresult != DS_OK)

		DS_Error(hresult, "DirectSound.SetCooperativeLevel");



	// Set up DSBUFFERDESC structure.

	memset(&dsbdesc, 0, sizeof(DSBUFFERDESC));	// Zero it out.

	dsbdesc.dwSize				= sizeof(DSBUFFERDESC);

	dsbdesc.dwFlags 			= DSBCAPS_PRIMARYBUFFER;

	dsbdesc.dwBufferBytes		= 0;

	dsbdesc.lpwfxFormat 		= NULL;



	hresult = lpDS->lpVtbl->CreateSoundBuffer(lpDS, &dsbdesc, &lpDSPrimary, NULL);

	if (hresult != DS_OK)

	{

		DS_Error(hresult, "DirectSound.CreateSoundBuffer - Primary");

	}



	// Set up wave format structure.

	memset( &wfx, 0, sizeof(WAVEFORMATEX) );

	wfx.wFormatTag		   = WAVE_FORMAT_PCM;	   

	wfx.nChannels		   = 2;

	wfx.nSamplesPerSec	   = 11025;

	wfx.nAvgBytesPerSec    = 11025*2*1;

	wfx.nBlockAlign 	   = 2; // ?

	wfx.wBitsPerSample	   = (WORD)8;

	wfx.cbSize			   = 0;



	hresult = lpDSPrimary->lpVtbl->SetFormat(lpDSPrimary, &wfx);

	if (hresult != DS_OK)

	{

		DS_Error(hresult, "DirectSound.SetFormat - Primary");

	}

	

	// Set the cooperative level so it doesn't get confused

	hresult = lpDS->lpVtbl->SetCooperativeLevel(lpDS, hMainWnd, DSSCL_NORMAL);

	if (hresult != DS_OK)

		DS_Error(hresult, "DirectSound.SetCooperativeLevel");

	

	return TRUE;

}



void ShutdownDirectSound()

{

	int buff;

	DWORD BufferStatus;



	for (buff = 0; buff < NUM_DSBUFFERS; buff++)

	{

		if (lpDSBuffer[buff] != 0)

		{

			BufferStatus = DSBSTATUS_PLAYING;

			while (BufferStatus == DSBSTATUS_PLAYING)

				lpDSBuffer[buff]->lpVtbl->GetStatus(lpDSBuffer[buff], &BufferStatus);

			RELEASE(lpDSBuffer[buff]);

		}

	}



	RELEASE(lpDSPrimary);

	RELEASE(lpDS);

}



static void CreateSoundBuffer(int Channel, int length, unsigned char *data)

{

	HRESULT 	   hresult;

	//int			 buff;

	DSBUFFERDESC   dsbdesc;

	PCMWAVEFORMAT  pcmwf;

	void		  *buffer, *buff2;

	DWORD		   size1, size2;



	if (Channel > NUM_DSBUFFERS)

	{

//		WriteDebug("Invalid sound effect...\n");

		return;

	}



	// Set up wave format structure.

	memset(&pcmwf, 0, sizeof(PCMWAVEFORMAT));

	pcmwf.wf.wFormatTag 		= WAVE_FORMAT_PCM;		

	pcmwf.wf.nChannels			= 1;

	pcmwf.wf.nSamplesPerSec 	= 8000;

	pcmwf.wf.nBlockAlign		= 1; // ?

	pcmwf.wf.nAvgBytesPerSec	= 8000*1*1;

	pcmwf.wBitsPerSample		= (WORD)8;

	

	// Set up DSBUFFERDESC structure.

	memset(&dsbdesc, 0, sizeof(DSBUFFERDESC));	// Zero it out.

	dsbdesc.dwSize				= sizeof(DSBUFFERDESC);

	dsbdesc.dwFlags 			= DSBCAPS_CTRLDEFAULT|DSBCAPS_CTRLPOSITIONNOTIFY;

	dsbdesc.dwBufferBytes		= length;

	dsbdesc.lpwfxFormat 		= (LPWAVEFORMATEX)&pcmwf;



	if ((hresult = lpDS->lpVtbl->CreateSoundBuffer(lpDS, &dsbdesc, &lpDSBuffer[Channel], NULL)) != DS_OK)

	{

		DS_Error(hresult, "DirectSound.CreateSoundBuffer");

		return;

	}

	

	hresult = lpDSBuffer[Channel]->lpVtbl->Lock(lpDSBuffer[Channel],0,length,&buffer,&size1,&buff2,&size2,DSBLOCK_ENTIREBUFFER );

	if (hresult == DS_OK)

	{

		memcpy(buffer, data, length);

		hresult = lpDSBuffer[Channel]->lpVtbl->Unlock(lpDSBuffer[Channel],buffer,length,buff2,size2);

		if (hresult != DS_OK)

			DS_Error(hresult, "lpDSBuffer.Unlock");

	}

	else

	{

		DS_Error(hresult, "lpDSBuffer.Lock");

	}

}



void S_LoadSound(int id, void* data, long length)

{

    HRESULT hresult;

    DWORD   dwDSBStatus;



	if (lpDSBuffer[id])

	{

		lpDSBuffer[id]->lpVtbl->GetStatus(lpDSBuffer[id], &dwDSBStatus);

		if (dwDSBStatus == DSBSTATUS_PLAYING)

		{

			hresult = lpDSBuffer[id]->lpVtbl->Stop(lpDSBuffer[id]);

			if (hresult != DS_OK)

				DS_Error(hresult, "lpDSBuffer.Stop");

		}

	}



	CreateSoundBuffer(id, length, data);

}



void S_PlaySound(int id, int pan)

{

    HRESULT hresult;

    DWORD   dwDSBStatus;



	lpDSBuffer[id]->lpVtbl->GetStatus(lpDSBuffer[id], &dwDSBStatus);

	if (dwDSBStatus == DSBSTATUS_PLAYING)

	{

		hresult = lpDSBuffer[id]->lpVtbl->Stop(lpDSBuffer[id]);

		if (hresult != DS_OK)

			DS_Error(hresult, "lpDSBuffer.Stop");

	}



	hresult = lpDSBuffer[id]->lpVtbl->SetCurrentPosition(lpDSBuffer[id], 0);

	if (hresult != DS_OK)

		DS_Error(hresult, "lpDSBuffer.SetCurrentPosition");

	hresult = lpDSBuffer[id]->lpVtbl->SetVolume(lpDSBuffer[id], 0);

	if (hresult != DS_OK)

		DS_Error(hresult, "lpDSBuffer.SetVolume");



	pan = pan*1000/15;

	hresult = lpDSBuffer[id]->lpVtbl->SetPan(lpDSBuffer[id], pan);

	if (hresult != DS_OK)

		DS_Error(hresult, "lpDSBuffer.SetPan");

	hresult = lpDSBuffer[id]->lpVtbl->Play(lpDSBuffer[id], 0, 0, 0);

	if (hresult != DS_OK)

		DS_Error(hresult, "lpDSBuffer.Play");

}



void S_UpdateSound()

{

}



void S_InitSound()

{ 

}

